#ifndef PTR_ARRAY_TEST_H_
#define PTR_ARRAY_TEST_H_

#include <cppunit/extensions/HelperMacros.h>
#include <alib/compare>

class PtrArrayTest : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE( PtrArrayTest );
  CPPUNIT_TEST( testProperties );
  CPPUNIT_TEST( testPolymorph );
  CPPUNIT_TEST_SUITE_END();

  enum class Type {
    CHILD1,
    CHILD2
  };

  class Base {
  public:
    virtual ~Base ( ) {
    }

    virtual Base * clone ( ) const = 0;
    virtual Base * plunder ( ) && = 0;

    virtual Type type ( ) const = 0;
  };

  class Child1 : public Base {
  public:
    virtual Base * clone ( ) const {
      return new Child1 ( * this );
    }

    virtual Base * plunder ( ) && {
      return new Child1 ( * this );
    }

    virtual Type type ( ) const {
      return Type::CHILD1;
    }
  };

  class Child2 : public Base {
  public:
    virtual Base * clone ( ) const {
      return new Child2 ( * this );
    }

    virtual Base * plunder ( ) && {
      return new Child2 ( * this );
    }

    virtual Type type ( ) const {
      return Type::CHILD2;
    }
  };

public:
  void setUp();
  void tearDown();

  void testProperties();
  void testPolymorph();
};

#endif  // PTR_ARRAY_TEST_H_
