#ifndef PTR_VECTOR_TEST_H_
#define PTR_VECTOR_TEST_H_

#include <cppunit/extensions/HelperMacros.h>
#include <alib/compare>

class PtrVectorTest : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE( PtrVectorTest );
  CPPUNIT_TEST( testProperties );
  CPPUNIT_TEST( testPolymorph );
  CPPUNIT_TEST( testRange );
  CPPUNIT_TEST_SUITE_END();

  enum class Type {
    CHILD1,
    CHILD2
  };

  class Base {
  public:
    virtual ~Base ( ) {
    }

    virtual Base * clone ( ) const = 0;
    virtual Base * plunder ( ) && = 0;

    virtual Type type ( ) const = 0;

    bool operator == ( const PtrVectorTest::Base & other ) const {
      return this->type ( ) == other.type ( );
    }

  };

  class Child1 : public Base {
  public:
    virtual Base * clone ( ) const {
      return new Child1 ( * this );
    }

    virtual Base * plunder ( ) && {
      return new Child1 ( * this );
    }

    virtual Type type ( ) const {
      return Type::CHILD1;
    }
  };

  class Child2 : public Base {
  public:
    virtual Base * clone ( ) const {
      return new Child2 ( * this );
    }

    virtual Base * plunder ( ) && {
      return new Child2 ( * this );
    }

    virtual Type type ( ) const {
      return Type::CHILD2;
    }
  };

class Moveable {
	int& m_moves;
	int& m_copies;

public:
	Moveable(int& moves, int& copies) : m_moves(moves), m_copies(copies) {
		m_moves = 0;
		m_copies = 0;
	}

	Moveable(const Moveable& src) : m_moves(src.m_moves), m_copies(src.m_copies) {
		m_copies++;
	}

	Moveable(Moveable&& src) : m_moves(src.m_moves), m_copies(src.m_copies) {
		m_moves++;
	}

	Moveable & operator = ( const Moveable & ) {
		m_copies ++;
		return * this;
	}

	Moveable & operator = ( Moveable && ) {
		m_moves ++;
		return * this;
	}

	bool operator<(const Moveable&) const {
		return false;
	}
};

public:
  void setUp();
  void tearDown();

  void testProperties();
  void testPolymorph();
  void testRange();
};

#endif  // PTR_VECTOR_TEST_H_
