#include "VectorTest.h"
#include <alib/vector>
#include <alib/type_traits>
#include <alib/algorithm>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( VectorTest, "bits" );
CPPUNIT_TEST_SUITE_REGISTRATION( VectorTest );

void VectorTest::setUp() {
}

void VectorTest::tearDown() {
}

void VectorTest::testProperties() {
	ext::vector<int> data = {1, 2, 3, 4};

	const ext::vector<int>& constData = data;

	CPPUNIT_ASSERT((std::is_same<decltype(constData[0]), const int&>::value));

	int moves;
	int copies;

	ext::vector < Moveable > vec;
	vec.push_back ( Moveable ( moves, copies ) );
	vec.push_back ( Moveable ( moves, copies ) );
	vec.push_back ( Moveable ( moves, copies ) );
	vec.push_back ( Moveable ( moves, copies ) );

	vec.erase ( ext::unique ( vec.begin ( ), vec.end ( ), [ ] ( const Moveable &, const Moveable & ) { return true; } ), vec.end ( ) );

	CPPUNIT_ASSERT ( vec.size ( ) == 1 );

	CPPUNIT_ASSERT(copies == 0);
}

void VectorTest::testVectorBool() {
	ext::vector < bool > A;
	A.push_back ( true );
	A.push_back ( false );
	A.push_back ( true );
	A.push_back ( false );

	ext::vector < bool > B;
	B.push_back ( true );
	B.push_back ( true );
	B.push_back ( false );
	B.push_back ( false );

	{
		ext::vector < bool > C = A | B;
		ext::vector < bool > D = A;
		D |= B;

		ext::vector < bool > X;
		X.push_back ( true );
		X.push_back ( true );
		X.push_back ( true );
		X.push_back ( false );

		std::cout << X << std::endl;
		std::cout << C << std::endl;

		CPPUNIT_ASSERT ( C == X );
		CPPUNIT_ASSERT ( D == X );
	}

	{
		ext::vector < bool > C = A & B;
		ext::vector < bool > D = A;
		D &= B;

		ext::vector < bool > X;
		X.push_back ( true );
		X.push_back ( false );
		X.push_back ( false );
		X.push_back ( false );

		CPPUNIT_ASSERT ( C == X );
		CPPUNIT_ASSERT ( D == X );
	}

	{
		ext::vector < bool > C = A ^ B;
		ext::vector < bool > D = A;
		D ^= B;

		ext::vector < bool > X;
		X.push_back ( false );
		X.push_back ( true );
		X.push_back ( true );
		X.push_back ( false );

		CPPUNIT_ASSERT ( C == X );
		CPPUNIT_ASSERT ( D == X );
	}
}

void testOffset ( long offset, long shift ) {
	unsigned long long shadow = 0x2A76B147D6521C87ULL;
	ext::vector < bool > data;

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 16 + offset; ++ i ) {
		data.push_back ( true );
	}

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	data >>= sizeof ( unsigned long long ) * 6 + shift;

	ext::vector < bool > ref;
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 10 + offset - shift; ++ i ) {
		ref.push_back ( true );
	}

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		ref.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	ref.resize ( 192 + offset );

/*	std::cout << "offset = " << offset << std::endl;
	std::cout << "shift = " << shift << std::endl;
	std::cout << "data = " << data << std::endl;
	std::cout << "ref  = " << ref << std::endl;*/

	CPPUNIT_ASSERT ( data == ref );
}

void VectorTest::testVectorBool2() {
	unsigned long long shadow = 0x2A76B147D6521C87ULL;
	ext::vector < bool > data;

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	shadow >>= 10;
	data >>= 10;

	ext::vector < bool > ref;
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		ref.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	std::cout << "data = " << data << std::endl;
	std::cout << "ref  = " << ref << std::endl;

	CPPUNIT_ASSERT ( data == ref );

	for ( long i = - ( long ) sizeof ( unsigned long long ) / 2 ; i < ( long ) sizeof ( unsigned long long ) * 8; ++ i )
		for ( long j = - ( long ) sizeof ( unsigned long long ) / 2 ; j < ( long ) sizeof ( unsigned long long ) * 8; ++ j )
			testOffset ( i, j );
}

void VectorTest::testVectorBool3() {
	unsigned long long shadow = 0x2A76B147D6521C87ULL;
	ext::vector < bool > data;

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	ext::vector < bool > data2 = data;
	data2 >>= 64;

	data >>= 32;
	data >>= 16;
	data >>= 8;
	data >>= 4;
	data >>= 2;
	data >>= 1;
	data >>= 1;

	std::cout << "data  = " << data  << std::endl;
	std::cout << "data2 = " << data2 << std::endl;

	data2 >>= 1;
	data2 >>= 2;
	data2 >>= 4;
	data2 >>= 8;
	data2 >>= 16;
	data2 >>= 32;
	data2 >>= 1;

	data >>= 27;
	data >>= 17;
	data >>= 13;
	data >>= 7;

	std::cout << "data  = " << data  << std::endl;
	std::cout << "data2 = " << data2 << std::endl;

	CPPUNIT_ASSERT ( data2 == data );
}

void testOffset2 ( long offset, long shift ) {
	unsigned long long shadow = 0x2A76B147D6521C87ULL;
	ext::vector < bool > data;

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8 + offset; ++ i ) {
		data.push_back ( true );
	}

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	data <<= sizeof ( unsigned long long ) * 2 + shift;

	ext::vector < bool > ref;
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 2 + shift; ++ i ) {
		ref.push_back ( false );
	}

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8 + offset; ++ i ) {
		ref.push_back ( true );
	}

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		ref.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	ref.resize ( 128 + offset );

/*	std::cout << "offset = " << offset << std::endl;
	std::cout << "shift = " << shift << std::endl;
	std::cout << "data = " << data << std::endl;
	std::cout << "ref  = " << ref << std::endl;*/

	CPPUNIT_ASSERT ( data == ref );
}

void VectorTest::testVectorBool4() {
	unsigned long long shadow = 0x2A76B147D6521C87ULL;
	ext::vector < bool > data;

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	shadow <<= 10;
	data <<= 10;

	ext::vector < bool > ref;
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		ref.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	std::cout << "data = " << data << std::endl;
	std::cout << "ref  = " << ref << std::endl;

	CPPUNIT_ASSERT ( data == ref );

	for ( long i = - ( long ) sizeof ( unsigned long long ) / 2 ; i < ( long ) sizeof ( unsigned long long ) * 8; ++ i )
		for ( long j = - ( long ) sizeof ( unsigned long long ) / 2 ; j < ( long ) sizeof ( unsigned long long ) * 8; ++ j )
			testOffset2 ( i, j );
}

void VectorTest::testVectorBool5() {
	unsigned long long shadow = 0x2A76B147D6521C87ULL;
	ext::vector < bool > data;

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	ext::vector < bool > data2 = data;
	data2 <<= 64;

	data <<= 32;
	data <<= 16;
	data <<= 8;
	data <<= 4;
	data <<= 2;
	data <<= 1;
	data <<= 1;

	std::cout << "data  = " << data  << std::endl;
	std::cout << "data2 = " << data2 << std::endl;

	data2 <<= 1;
	data2 <<= 2;
	data2 <<= 4;
	data2 <<= 8;
	data2 <<= 16;
	data2 <<= 32;
	data2 <<= 1;

	data <<= 27;
	data <<= 17;
	data <<= 13;
	data <<= 7;

	std::cout << "data  = " << data  << std::endl;
	std::cout << "data2 = " << data2 << std::endl;

	CPPUNIT_ASSERT ( data2 == data );
}

void VectorTest::testVectorBool6 ( ) {
	for ( size_t i = 1; i < 1000; i += 31 ) {
		for ( size_t j = 1; j < 1000; j += 31 ) {
			ext::vector < bool > first;
			ext::vector < bool > second;

			first.resize ( i );
			second.resize ( j );

			ext::fill ( first );
			ext::clear ( second );

			ext::vector < bool > res = first & second;

			CPPUNIT_ASSERT ( ext::any ( res ) == false );
		}
	}
}

void VectorTest::testRange ( ) {
	ext::vector < int > data { 1, 2, 3, 4 };
	ext::vector < int > data2 ( data.range ( ) );

	CPPUNIT_ASSERT ( data == data2 );

	int moves;
	int copies;

	ext::vector < Moveable > vec;
	vec.push_back ( Moveable ( moves, copies ) );
	vec.push_back ( Moveable ( moves, copies ) );
	vec.push_back ( Moveable ( moves, copies ) );
	vec.push_back ( Moveable ( moves, copies ) );

	ext::vector < Moveable > vec2 ( ext::range ( std::move ( vec ) ) );

	CPPUNIT_ASSERT ( copies == 0 );
}
