#include "ManagedLinearSetTest.h"
#include <exception>

#include <alib/managed_linear_set>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( ManagedLinearSetTest, "bits" );
CPPUNIT_TEST_SUITE_REGISTRATION ( ManagedLinearSetTest );

void ManagedLinearSetTest::setUp ( ) {
}

void ManagedLinearSetTest::tearDown ( ) {
}

void ManagedLinearSetTest::testBasics ( ) {
	ext::managed_linear_set < int > mod2;
	mod2.addInsertCallback ( [](const int & value) { if ( value % 2 != 0 ) throw value; } );
	CPPUNIT_ASSERT_NO_THROW ( mod2.insert ( 12 ) );
	CPPUNIT_ASSERT_THROW ( mod2.insert ( 13 ), int );

	ext::managed_linear_set < int > mod6;
	mod6.addInsertCallback ( [](const int & value) { if ( value % 6 != 0 ) throw value; } );
	CPPUNIT_ASSERT_NO_THROW ( mod6.insert ( 6 ) );
	CPPUNIT_ASSERT_THROW ( mod6.insert ( 4 ), int );

	CPPUNIT_ASSERT_NO_THROW ( std::swap ( mod2, mod6 ) );

	mod2.insert ( 4 );

	mod2.addInsertCallback ( [](const int & value) { std::cout << "mod2 added: " << value << std::endl; } );
	mod2.addRemoveCallback ( [](const int & value) { std::cout << "mod2 removed: " << value << std::endl; } );
	mod6.addInsertCallback ( [](const int & value) { std::cout << "mod6 added: " << value << std::endl; } );
	mod6.addRemoveCallback ( [](const int & value) { std::cout << "mod6 removed: " << value << std::endl; } );

	std::cout << "mod2: " << mod2 << std::endl;
	std::cout << "mod6: " << mod6 << std::endl;
	CPPUNIT_ASSERT_THROW ( std::swap ( mod2, mod6 ), int );
	std::cout << "mod2: " << mod2 << std::endl;
	std::cout << "mod6: " << mod6 << std::endl;
	CPPUNIT_ASSERT_THROW ( swap ( mod2, mod6 ), int );
	std::cout << "mod2: " << mod2 << std::endl;
	std::cout << "mod6: " << mod6 << std::endl;

	CPPUNIT_ASSERT_NO_THROW ( mod2.insert ( mod6.begin ( ), mod6.end ( ) ) );
	CPPUNIT_ASSERT_THROW ( mod2.insert ( { 8, 9 } ), int );

	std::cout << "mod2: " << mod2 << std::endl;
	std::cout << "mod6: " << mod6 << std::endl;

	mod2 = { 4, 6, 8 };
}
